@@ -10,8 +10,8 @@ Created on Wed Apr  1 14:56:15 2020
 """
 import pkg_resources
 import numpy as np
-from .InputHipims import InputHipims
-from .Raster import Raster
+#from .InputHipims import InputHipims
+#from .Raster import Raster
 
 def demo_input(set_example_inputs=True):
     """ A demonstration to generate a hipims input object
